import java.io.*;

/** EnchantingMessenger provides two-way communication between the
    Dispatcher and Enchanting or Scratch (or Scratch-derivatives). */
public class EnchantingMessenger extends Messenger {

	/** The port (on localhost) to try to connect to. */
	int port;
	
	ScratchIO io;
	
	public EnchantingMessenger(String name, Dispatcher dispatcher, int connectionDelay, int verbosity, int port) {
		super(name, dispatcher, connectionDelay, verbosity);
		this.port = port;
	}
	
	public void connect() {
		while (io == null)
		{
			try {
				sleep(connectionDelay);
				io = new ScratchIO(port);
			}
			catch (InterruptedException e) {
				// we are unlikely to be interrupted
				throw new RuntimeException(e);
			}
			catch (IOException e) {
				// We failed to connect.
				// Try again in a moment.
				;
			}
		}
	}
	
	public void receiveMessages() {
		while (io != null) {
			try {
				ScratchMessage msg = io.readMsg();
				if (msg == null) {
					io = null;	// request reconnect
				} else {
					dispatcher.forward(msg, this);
				}
				
			}
			catch (IOException e) {
				io = null;	// request reconnect
			}
		}
	}
	
	public boolean sendMessage(ScratchMessage msg, Messenger source) {
		if (connected) {
			try { io.sendMsg(msg); }
			catch (IOException e) { return false; }
			return true;
		}
		return false;
	}
}
