Object subclass: #EnchantingDeviceConfiguration
	instanceVariableNames: ''
	classVariableNames: 'BackEndConnected DeviceConnected DeviceReady '
	poolDictionaries: ''
	category: 'Enchanting-Configuration'!
!EnchantingDeviceConfiguration commentStamp: '<historical>' prior: 0!
The EnchantingDeviceConfiguration stores all the information about how a device is configured.  It is an abstract base class.  The configuration is edited through the use of the EnchantingConfigurationDialog.!


!EnchantingDeviceConfiguration methodsFor: 'accessing' stamp: 'CWB 12/17/2010 11:14'!
configuredPeripherals
"Returns a list of all the configured peripherals"

	self subclassResponsibility.! !

!EnchantingDeviceConfiguration methodsFor: 'accessing' stamp: 'CWB 12/17/2010 11:15'!
peripheralNamed: aName
"Returns the (first) peripheral with the given name."

	^ self configuredPeripherals detect: [:p | p name = aName] ifNone: [ nil ]. ! !

!EnchantingDeviceConfiguration methodsFor: 'accessing' stamp: 'CWB 12/11/2010 20:12'!
setOfConfiguredClasses
"Returns a set of all the configured classes"

	self subclassResponsibility.! !


!EnchantingDeviceConfiguration methodsFor: 'block specs' stamp: 'CWB 9/5/2012 06:02'!
activeBlockSpecsFor: blockSpecRequester

	"This function is similar to ScratchSpriteMorph>>blockSpecs;
	it returns a list of all blocks that should be available to the user
	based on which peripherals are configured."

	"The blockSpecRequester can be #sprite, #stage, or #all."

	| allPeripherals activePeripherals orderedPeripherals blockSpecs |

	allPeripherals _ self class orderedPeripheralClasses.
	activePeripherals _ self setOfConfiguredClasses.

	"Get a list of all the configured peripherals in a the correct order."
	orderedPeripherals _ allPeripherals select: [:p | p providesBlocksUsedBy: activePeripherals].

	blockSpecs _ OrderedCollection new.	
	orderedPeripherals do: [:p | blockSpecs addAll: p providesBlocks; add: #-].

	blockSpecs addAll: (self class deviceBlockSpecsFor: blockSpecRequester).

	^ blockSpecs.

			
	

	! !


!EnchantingDeviceConfiguration methodsFor: 'dialog interaction' stamp: 'CWB 1/1/2011 16:36'!
applyChangesFrom: dialog named: dialogName over: scratchFrame
"The user pressed okay on the dialog; update structures to reflect the changes."

	self subclassResponsibility.! !

!EnchantingDeviceConfiguration methodsFor: 'dialog interaction' stamp: 'CWB 12/4/2010 11:17'!
portsFor: dialogName
"Returns an array containing:
	an array with the names of all the ports
	an array (of the same size) of ConfigurationBlockMorphs (or nil) for each port."

	self subclassResponsibility.! !

!EnchantingDeviceConfiguration methodsFor: 'dialog interaction' stamp: 'CWB 1/1/2011 21:33'!
replaceNamesInScriptsUsing: aNameDictionary over: scratchFrame
"With a dictionary that maps old names onto new names (or nil, for deleted items), this function replaces the names in all blocks."

	| knownNames |

	knownNames _ aNameDictionary keys.

	scratchFrame stageAndSprites do: [:s |
		(s blocksBin isKindOf: Morph) ifTrue: [
			| todo |
			todo _ OrderedCollection new.
			todo addAll: s blocksBin submorphs.	"A list of stacks or top blocks, not all starting with a hat morph"
			[ todo isEmpty ] whileFalse: [
				| morph |
				morph _ todo removeFirst.

				(morph isKindOf: BlockMorph) ifTrue: [

					todo addAll: morph submorphs.

					"Is this a block that uses a peripheral's name?"
					((morph isKindOf: CommandBlockMorph) 
					and: [morph commandSpec includesSubString: '%P']) 
					ifTrue: [
						| choiceMorph peripheralName |
						choiceMorph _ morph submorphs detect: [:m | 
							(m isKindOf: ChoiceArgMorphForPeripheral) 
							and: [ m getOptionsSelector = #applicablePeripheralNamesFor: ] 
						].
						peripheralName _ choiceMorph choice.
						
						(knownNames includes: peripheralName) ifTrue: [
							| newName |
							newName _ aNameDictionary at: peripheralName.
							newName ifNotNil: [
								choiceMorph choice: newName.
							] ifNil: [
								choiceMorph choice: choiceMorph class invalidContents.
							].
						].
					].
				].
			].
		]
	].
! !

!EnchantingDeviceConfiguration methodsFor: 'dialog interaction' stamp: 'CWB 1/1/2011 16:36'!
runConfigurationDialog: dialogName over: scratchFrame
"Displays a specific configuration dialog"

	| dlg morphs blockColor result |

	dlg _ EnchantingConfigurationDialog new.
	blockColor _ self class blockColorFor: dialogName.
	morphs _ (self class peripheralsFor: dialogName) collect: 
	[ :peripheralClass |
		peripheralClass createGenericConfigBlock
			color: blockColor
	].
	dlg buildPaletteWith: morphs.
	dlg buildBlockBinWith: (self portsFor: dialogName) in: blockColor.
	dlg namesInUse: (self configuredPeripherals collect: [:p | p name]).

	result _ dlg openInWorld getUserResponse.
	(result = #cancelled) ifFalse: [ self applyChangesFrom: dlg named: dialogName over: scratchFrame ].
! !


!EnchantingDeviceConfiguration methodsFor: 'dropdown lists' stamp: 'CWB 1/1/2011 19:43'!
applicablePeripheralNamesFor: aBlock
"Returns a list of names that apply for the given peripheral.  For example, if this is a light sensor block, and there are three light sensors configured, this will return all of their names."

	| peripheralClass applicablePeripherals result |

	peripheralClass _ aBlock peripheralClass.
	peripheralClass ifNil: [^ #('error')].

	"Go through all peripherals of this class, and put their name in an array"
	applicablePeripherals _ self configuredPeripherals select: [:p | p class actsAs: peripheralClass].
	result _ applicablePeripherals collect: [:p | p name].
	(result size < 1) ifTrue: [
		^ OrderedCollection with: ChoiceArgMorphForPeripheral invalidContents.
		"self error: 'No applicable peripheral names for request'"
	].
	^ result.	! !

!EnchantingDeviceConfiguration methodsFor: 'dropdown lists' stamp: 'CWB 12/18/2010 03:41'!
concreteSubtypesFor: aBlock
"Returns a list of the sensors that implement the features used in this block."

	| peripheralClass |

	peripheralClass _ aBlock peripheralClass.
	peripheralClass ifNil: [^ #('error')].

	^ peripheralClass concreteTypes.
! !

!EnchantingDeviceConfiguration methodsFor: 'dropdown lists' stamp: 'CWB 12/16/2010 20:06'!
peripheralCustomList1For: aBlock
	
	^  self peripheralCustomList: 1 for: aBlock
! !

!EnchantingDeviceConfiguration methodsFor: 'dropdown lists' stamp: 'CWB 12/16/2010 20:06'!
peripheralCustomList2For: aBlock
	
	^  self peripheralCustomList: 2 for: aBlock
! !

!EnchantingDeviceConfiguration methodsFor: 'dropdown lists' stamp: 'CWB 12/16/2010 20:06'!
peripheralCustomList3For: aBlock
	
	^  self peripheralCustomList: 3 for: aBlock
! !

!EnchantingDeviceConfiguration methodsFor: 'dropdown lists' stamp: 'CWB 12/16/2010 20:46'!
peripheralCustomList: aNumber for: aBlock
"Peripherals can return custom lists for a combo box that shows up on their block."

	| peripheralClass result |

	result _ nil.
	peripheralClass _ aBlock peripheralClass.
	peripheralClass ifNotNil: [
		result _ peripheralClass customListNumber: aNumber
	].

	^ result ifNotNil: [ result ] ifNil: [ #('error') ].
! !


!EnchantingDeviceConfiguration methodsFor: 'exporting' stamp: 'CWB 12/17/2010 02:12'!
emitInitializationCodeOn: aStream for: anExporter
"Outputs the code to initialize the various devices."

"Basic version here goes through all devices and forwards the request to output the code."

	self configuredPeripherals do: [:p |
		p emitInitializationCodeOn: aStream for: anExporter.
	].! !


!EnchantingDeviceConfiguration methodsFor: 'object i/o' stamp: 'CWB 12/7/2010 20:43'!
fieldsVersion

	^ 1! !

!EnchantingDeviceConfiguration methodsFor: 'object i/o' stamp: 'CWB 12/7/2010 20:42'!
initFieldsFrom: anObjStream version: classVersion

	self initFieldsNamed: #( ) from: anObjStream.! !

!EnchantingDeviceConfiguration methodsFor: 'object i/o' stamp: 'CWB 12/7/2010 20:43'!
storeFieldsOn: anObjStream

	self storeFieldsNamed: #( ) on: anObjStream.! !


!EnchantingDeviceConfiguration methodsFor: 'other' stamp: 'CWB 11/12/2011 16:47'!
customExportColor: aColor for: aBlock on: aStream with: anExporter
"Export a color in some way that makes sense for your device."

	^ self subclassResponsibility.! !

!EnchantingDeviceConfiguration methodsFor: 'other' stamp: 'CWB 5/7/2012 19:55'!
exportSprite: aSprite on: aStream for: anExporter
	self inform: 'Exporting a sprite for current external device is not supported'! !

!EnchantingDeviceConfiguration methodsFor: 'other' stamp: 'MJM 3/27/2011 00:42'!
uncoloredArgMorphFor: specString

	"Allows the NXT device to create an argument other than that created by CommandBlockMorph."

	"Returns nil or an argMorph."

	self subclassResponsibility.
! !


!EnchantingDeviceConfiguration methodsFor: 'display' stamp: 'CWB 5/12/2012 13:20'!
mapFromDisplayToScreenSpace: point
	"Take a point that is specified in the coordinate space for your display,
	and return a corresponding point relative to the center of the on-screen stage."

	^ point.! !

!EnchantingDeviceConfiguration methodsFor: 'display' stamp: 'CWB 5/12/2012 13:20'!
mapFromScreenToDisplaySpace: point
	"Take a point that is specified relative to the center of the on-screen stage,
	and return the corresponding coordinate suitable for the device at hand."

	^ point.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

EnchantingDeviceConfiguration class
	instanceVariableNames: ''!

!EnchantingDeviceConfiguration class methodsFor: 'accessing' stamp: 'CWB 12/11/2010 20:03'!
orderedPeripheralClasses

	"Returns a list of all peripherals that might be set up for this device.
	The order is important, as it will be followed when placing blocks in the
	palettes in the main user interface.  (In other words, if you return 
	{ WhizBang . SawBuzz } the WhizBang blocks will always appear before
	the SawBuzz blocks."

	self subclassResponsibility.! !


!EnchantingDeviceConfiguration class methodsFor: 'block specs' stamp: 'CWB 9/5/2012 06:02'!
allBlockSpecs
"Returns a list of all possible block specs this device can use."

	| allSpecs |
	
	allSpecs _ OrderedCollection new.
	self orderedPeripheralClasses do: [:p | 
		allSpecs addAll: (p providesBlocks).
		allSpecs addAll: (p obsoleteBlocks).
	].
	allSpecs addAll: (self deviceBlockSpecsFor: #all).
	allSpecs addAll: (self obsoleteDeviceBlockSpecs).
	^ allSpecs.! !

!EnchantingDeviceConfiguration class methodsFor: 'block specs' stamp: 'CWB 9/5/2012 06:02'!
blockSpecsForTranslation
"Returns a list of all non-obsolete block specs this device can use.  (Very similar to allBlockSpecs)"

	| specs configSpec specStrings |
	
	specs _ OrderedCollection new.
	self orderedPeripheralClasses do: [:p | 
		specs addAll: (p providesBlocks).
		(configSpec _ p configBlockSpec) ifNotNil: [ specs add: configSpec ].
	].
	specs addAll: (self deviceBlockSpecsFor: #all).

	
	"We really only want the block specifications or category titles out of the list"
	specStrings _ Set new.

	specs do: [:el |
		(el isKindOf: Array)
			ifTrue: [specStrings add: el first]
			ifFalse: [((el beginsWith: 'obsolete') or:[#('-' '~') includes: el])
				ifFalse:[specStrings add: el]]].

	^ specStrings.


! !

!EnchantingDeviceConfiguration class methodsFor: 'block specs' stamp: 'CWB 12/17/2010 10:29'!
deviceBlockSpecs
"Override this to supply block specs that should always be active for this device."

	^ self standardBlockSpecs! !

!EnchantingDeviceConfiguration class methodsFor: 'block specs' stamp: 'CWB 12/21/2012 19:44'!
obsoleteDeviceBlockSpecs
"A list of blocks that were in deviceBlockSpecs but have since been deprecated."

	^ #(
		'obsolete sprite motion blocks'
 			('turn %n degrees'				-	turnRight: 15)	"icon shows turn direction"
 			('turn %n degrees'				-	turnLeft: 15)	"icon shows turn direction"
	).! !

!EnchantingDeviceConfiguration class methodsFor: 'block specs' stamp: 'CWB 8/8/2011 10:13'!
standardBlockSpecs
"Returns block specs that should always be active.  (Override if necessary)."

	"By default, return control and variable blocks, but ignore sound and motor blocks"

	| stageBlocks standardBlocks permitList allow |

	stageBlocks _ ScriptableScratchMorph blockSpecs.
	standardBlocks _ OrderedCollection new.
	permitList _ #('control', 'variables', 'operators', 'list').

	stageBlocks do: [ :b |
		"A list of blockspecs has strings for titles (and 1-character strings for spaces)"
		"Use the title of a section to see if we want to include it or not"
		((b isKindOf: String) and: [b size > 1]) ifTrue: [ allow _ permitList includes: b ].
		allow ifTrue: [ standardBlocks add: b ].
	].
	
	^ standardBlocks.! !


!EnchantingDeviceConfiguration class methodsFor: 'dialog interaction' stamp: 'CWB 1/1/2011 15:36'!
blockColorFor: dialogName
"Returns the color ConfigurationBlockMorphs should be in the specified dialog"
	
	^ ScriptableScratchMorph blockColorFor: dialogName.
! !

!EnchantingDeviceConfiguration class methodsFor: 'dialog interaction' stamp: 'CWB 11/20/2010 15:12'!
peripheralsFor: dialogName
'Returns a list of all the peripherals for the specified dialog'

	self subclassResponsibility! !

!EnchantingDeviceConfiguration class methodsFor: 'dialog interaction' stamp: 'CWB 12/4/2010 11:12'!
portName: baseName at: count method: countMethod
"Returns a name for a port."
"countMethod can be #numeric, #alphabetical, or #none"

	| countStr |

	countStr _ ''.
	(countMethod = #numeric) ifTrue: [countStr _ ' ', count asString].
	(countMethod = #alphabetical) ifTrue: [countStr _ ' ', (64 + count) asCharacter asString].

	"Assumption: it is always appropriate to end with the port number"
	^ baseName localized, countStr.! !


!EnchantingDeviceConfiguration class methodsFor: 'connection details' stamp: 'CWB 5/28/2011 17:57'!
activeCommandButtons
"should the green flag and stop buttons be active?"

	^ (BackEndConnected and: [DeviceConnected]) and: [DeviceReady].! !

!EnchantingDeviceConfiguration class methodsFor: 'connection details' stamp: 'CWB 5/28/2011 15:30'!
backEndConnected
"Is the back end connected?"

	^ BackEndConnected.! !

!EnchantingDeviceConfiguration class methodsFor: 'connection details' stamp: 'CWB 5/28/2011 15:30'!
backEndConnected: isConnected
"Is the back end connected?"

	^ BackEndConnected _ isConnected.! !

!EnchantingDeviceConfiguration class methodsFor: 'connection details' stamp: 'CWB 5/28/2011 15:31'!
deviceConnected
"Is the device connected?"

	^ DeviceConnected.! !

!EnchantingDeviceConfiguration class methodsFor: 'connection details' stamp: 'CWB 5/28/2011 15:31'!
deviceConnected: isConnected
"Is the device connected?"

	^ DeviceConnected _ isConnected.! !

!EnchantingDeviceConfiguration class methodsFor: 'connection details' stamp: 'CWB 5/28/2011 15:32'!
deviceReady
"Is the device ready to accept commands?"

	^ DeviceReady.! !

!EnchantingDeviceConfiguration class methodsFor: 'connection details' stamp: 'CWB 5/28/2011 15:32'!
deviceReady: isReady
"Is the device ready for commands?"

	^ DeviceReady _ isReady.! !

!EnchantingDeviceConfiguration class methodsFor: 'connection details' stamp: 'CWB 5/31/2011 06:20'!
deviceStatusMessage

	BackEndConnected ifTrue: [
		DeviceConnected ifTrue: [
			DeviceReady ifTrue: [
				^ 'Device is ready.'
			] ifFalse: [
				^ 'Device is busy.'
			].
		] ifFalse: [
			^ 'No device connected.'
		].
	] ifFalse: [
		^ 'No communication with BackEnd'.
	].! !
